==> 1.2.1.tar.gz: lib/src/application/application_impl.dart <==
part of 'application.dart';

/// Application root widget
class Application extends StatefulWidget {
  final String name;
  final String initialRoute;
  final Set<ActivityRoute> routes;
  final OnUnknownRoute onUnknownRoute;
  final Set<LabelInterface>? labels;
  final ValueSetter<LabelInterface>? onLabel;
  final AppFactory appFactory;

  const Application._(
      {Key? key,
      required this.name,
      required this.initialRoute,
      required this.routes,
      required this.onUnknownRoute,
      this.labels,
      this.onLabel,
      required this.appFactory})
      : assert(labels == null || onLabel != null, 'onLabel must be provided along with labels'),
        super(key: key);

  @override
  ApplicationState createState() => ApplicationState();

  static ApplicationState of(BuildContext context) =>
      requireNotNull(context.findRootAncestorStateOfType<ApplicationState>(),
          message: () => 'Application not reachable from current context. Have you called runApplication()?');

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties
      ..add(StringProperty('name', name))
      ..add(StringProperty('initialRoute', initialRoute))
      ..add(IterableProperty<ActivityRoute>('routes', routes))
      ..add(ObjectFlagProperty<OnUnknownRoute>.has('onUnknownRoute', onUnknownRoute))
      ..add(IterableProperty<LabelInterface>('labels', labels))
      ..add(ObjectFlagProperty<ValueSetter<LabelInterface>?>.has('onLabel', onLabel))
      ..add(ObjectFlagProperty<AppFactory>.has('appFactory', appFactory));
  }
}

class ApplicationState extends State<Application> {
  GlobalKey _key = GlobalKey();

  final LiveEvent<void> onDestroy = LiveEvent();

  final MutableLiveData<bool> isInputMethodVisible = MutableLiveData(initialValue: false);

  final MutableLiveData<AppLifecycleState> lifecycle =
      MutableLiveData(initialValue: SchedulerBinding.instance!.lifecycleState);

  late String _title;

  String get title => _title;

  set title(String? value) => SystemChrome.setApplicationSwitcherDescription(ApplicationSwitcherDescription(
      label: _title = value ?? widget.name, primaryColor: Theme.of(context).primaryColor.value));

  List<Locale> _supportedLocales = const [];

  List<Locale> get supportedLocales => _supportedLocales;

  late Locale _defaultLocale;

  Locale get defaultLocale => _defaultLocale;

  Locale? _enforcedLocale;

  Locale? get enforcedLocale => _enforcedLocale;

  set enforcedLocale(Locale? value) =>
      WidgetsBinding.instance!.endOfFrame.then((_) => setState(() => _enforcedLocale = value));

  late _AppBindingObserver _binding;

  late AppRouterDelegate _appRouterDelegate;

  void restart() => WidgetsBinding.instance!.endOfFrame.then((_) {
        onDestroy.send(null);
        sessionStorage.clear();
        _GlobalViewModelProvider.instance.clearViewModels();
        _appRouterDelegate = AppRouterDelegate(this, widget.routes, widget.initialRoute, widget.onUnknownRoute);
        setState(() => _key = GlobalKey());
      });

  @override
  void initState() {
    super.initState();
    _title = widget.name;
    _binding = _AppBindingObserver(this);
    _appRouterDelegate = AppRouterDelegate(this, widget.routes, widget.initialRoute, widget.onUnknownRoute);
  }

  @override
  Widget build(BuildContext context) {
    String onGenerateTitle(BuildContext _) => title;
    final labels = widget.labels;
    final labelInterfaceDelegate = labels == null ? null : LabelInterfaceDelegate(labels, widget.onLabel!);
    final localizationsDelegates = <LocalizationsDelegate>[
      if (labelInterfaceDelegate != null) labelInterfaceDelegate,
      GlobalMaterialLocalizations.delegate,
      GlobalWidgetsLocalizations.delegate,
      GlobalCupertinoLocalizations.delegate,
    ];
    final systemLocales = WidgetsBinding.instance!.window.locales;
    _supportedLocales = labels?.map((label) => label.locale).toList(growable: false) ?? systemLocales;
    _defaultLocale = basicLocaleListResolution(systemLocales, _supportedLocales);
    final root = widget.appFactory(_key, _appRouterDelegate.provider, _appRouterDelegate.parser, _appRouterDelegate,
        onGenerateTitle, _enforcedLocale, localizationsDelegates, _supportedLocales);
    assert(() {
      WidgetsBinding.instance!.addPostFrameCallback((_) {
        final app = _findWidgetsApp(_key);
        if (app.routeInformationProvider != _appRouterDelegate.provider ||
            app.routeInformationParser != _appRouterDelegate.parser ||
            app.routerDelegate != _appRouterDelegate ||
            app.onGenerateTitle != onGenerateTitle ||
            (labelInterfaceDelegate != null && app.localizationsDelegates?.contains(labelInterfaceDelegate) != true) ||
            (labels != null && !app.supportedLocales.toSet().containsAll(_supportedLocales)))
          throw StateError('MaterialAppFactory must set all given parameters');
      });
      return true;
    }(), 'WidgetsApp parameters check');
    return root;
  }

  WidgetsApp _findWidgetsApp(GlobalKey key) {
    final app = key.currentWidget;
    if (app is WidgetsApp) return app;
    key = GlobalObjectKey(requireNotNull(_key.currentState,
        message: () =>
            'appFactory must provide key parameter to one of the following widgets: MaterialApp | CupertinoApp | WidgetsApp'));
    return _findWidgetsApp(key);
  }

  @override
  void dispose() {
    onDestroy.send(null);
    _binding.dispose();
    super.dispose();
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties
      ..add(DiagnosticsProperty<LiveEvent<void>>('onDestroy', onDestroy))
      ..add(DiagnosticsProperty<MutableLiveData<bool>>('isInputMethodVisible', isInputMethodVisible))
      ..add(DiagnosticsProperty<MutableLiveData<AppLifecycleState>>('lifecycle', lifecycle))
      ..add(StringProperty('title', title))
      ..add(IterableProperty<Locale>('supportedLocales', supportedLocales))
      ..add(DiagnosticsProperty<Locale>('defaultLocale', defaultLocale))
      ..add(DiagnosticsProperty<Locale?>('enforcedLocale', enforcedLocale));
  }
}

==> 1.2.4.tar.gz: lib/src/application/application_impl.dart <==
part of 'application.dart';

/// Application root widget
class Application extends StatefulWidget {
  final String name;
  final String initialRoute;
  final Set<ActivityRoute> routes;
  final OnUnknownRoute onUnknownRoute;
  final Set<LabelInterface>? labels;
  final ValueSetter<LabelInterface>? onLabel;
  final AppFactory appFactory;

  const Application._(
      {Key? key,
      required this.name,
      required this.initialRoute,
      required this.routes,
      required this.onUnknownRoute,
      this.labels,
      this.onLabel,
      required this.appFactory})
      : assert(labels == null || onLabel != null, 'onLabel must be provided along with labels'),
        super(key: key);

  @override
  ApplicationState createState() => ApplicationState();

  static ApplicationState of(BuildContext context) =>
      requireNotNull(context.findRootAncestorStateOfType<ApplicationState>(),
          message: () => 'Application not reachable from current context. Have you called runApplication()?');

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties
      ..add(StringProperty('name', name))
      ..add(StringProperty('initialRoute', initialRoute))
      ..add(IterableProperty<ActivityRoute>('routes', routes))
      ..add(ObjectFlagProperty<OnUnknownRoute>.has('onUnknownRoute', onUnknownRoute))
      ..add(IterableProperty<LabelInterface>('labels', labels))
      ..add(ObjectFlagProperty<ValueSetter<LabelInterface>?>.has('onLabel', onLabel))
      ..add(ObjectFlagProperty<AppFactory>.has('appFactory', appFactory));
  }
}

class ApplicationState extends State<Application> {
  GlobalKey _key = GlobalKey();

  final LiveEvent<void> onDestroy = LiveEvent();

  final MutableLiveData<bool> isInputMethodVisible = MutableLiveData(initialValue: false);

  final MutableLiveData<AppLifecycleState> lifecycle =
      MutableLiveData(initialValue: SchedulerBinding.instance.lifecycleState);

  late String _title;

  String get title => _title;

  set title(String? value) => SystemChrome.setApplicationSwitcherDescription(ApplicationSwitcherDescription(
      label: _title = value ?? widget.name, primaryColor: Theme.of(context).primaryColor.value));

  List<Locale> _supportedLocales = const [];

  List<Locale> get supportedLocales => _supportedLocales;

  late Locale _defaultLocale;

  Locale get defaultLocale => _defaultLocale;

  Locale? _enforcedLocale;

  Locale? get enforcedLocale => _enforcedLocale;

  set enforcedLocale(Locale? value) => Dispatcher.postFrame.post(() => setState(() => _enforcedLocale = value));

  late _AppBindingObserver _binding;

  late AppRouterDelegate _appRouterDelegate;

  void restart() => Dispatcher.postFrame.post(() {
        onDestroy.send(null);
        sessionStorage.clear();
        _GlobalViewModelProvider.instance.clearViewModels();
        _appRouterDelegate = AppRouterDelegate(this, widget.routes, widget.initialRoute, widget.onUnknownRoute);
        setState(() => _key = GlobalKey());
      });

  @override
  void initState() {
    super.initState();
    _title = widget.name;
    _binding = _AppBindingObserver(this);
    _appRouterDelegate = AppRouterDelegate(this, widget.routes, widget.initialRoute, widget.onUnknownRoute);
  }

  @override
  Widget build(BuildContext context) {
    String onGenerateTitle(BuildContext _) => title;
    final labels = widget.labels;
    final labelInterfaceDelegate = labels == null ? null : LabelInterfaceDelegate(labels, widget.onLabel!);
    final localizationsDelegates = <LocalizationsDelegate>[
      if (labelInterfaceDelegate != null) labelInterfaceDelegate,
      GlobalMaterialLocalizations.delegate,
      GlobalWidgetsLocalizations.delegate,
      GlobalCupertinoLocalizations.delegate,
    ];
    final systemLocales = WidgetsBinding.instance.window.locales;
    _supportedLocales = labels?.map((label) => label.locale).toList(growable: false) ?? systemLocales;
    _defaultLocale = basicLocaleListResolution(systemLocales, _supportedLocales);
    final root = widget.appFactory(_key, _appRouterDelegate.provider, _appRouterDelegate.parser, _appRouterDelegate,
        onGenerateTitle, _enforcedLocale, localizationsDelegates, _supportedLocales);
    assert(() {
      Dispatcher.postFrame.post(() {
        final app = _findWidgetsApp(_key);
        if (app.routeInformationProvider != _appRouterDelegate.provider ||
            app.routeInformationParser != _appRouterDelegate.parser ||
            app.routerDelegate != _appRouterDelegate ||
            app.onGenerateTitle != onGenerateTitle ||
            (labelInterfaceDelegate != null && app.localizationsDelegates?.contains(labelInterfaceDelegate) != true) ||
            (labels != null && !app.supportedLocales.toSet().containsAll(_supportedLocales)))
          throw StateError('MaterialAppFactory must set all given parameters');
      });
      return true;
    }(), 'WidgetsApp parameters check');
    return root;
  }

  WidgetsApp _findWidgetsApp(GlobalKey key) {
    final app = key.currentWidget;
    if (app is WidgetsApp) return app;
    key = GlobalObjectKey(requireNotNull(_key.currentState,
        message: () =>
            'appFactory must provide key parameter to one of the following widgets: MaterialApp | CupertinoApp | WidgetsApp'));
    return _findWidgetsApp(key);
  }

  @override
  void dispose() {
    onDestroy.send(null);
    _binding.dispose();
    super.dispose();
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties
      ..add(DiagnosticsProperty<LiveEvent<void>>('onDestroy', onDestroy))
      ..add(DiagnosticsProperty<MutableLiveData<bool>>('isInputMethodVisible', isInputMethodVisible))
      ..add(DiagnosticsProperty<MutableLiveData<AppLifecycleState>>('lifecycle', lifecycle))
      ..add(StringProperty('title', title))
      ..add(IterableProperty<Locale>('supportedLocales', supportedLocales))
      ..add(DiagnosticsProperty<Locale>('defaultLocale', defaultLocale))
      ..add(DiagnosticsProperty<Locale?>('enforcedLocale', enforcedLocale));
  }
}
